import 'package:provider/provider.dart';
import 'package:ui_sdk/json.dart';
import 'package:ui_sdk/props/StandardScreenResponse.dart';
import 'package:ui_sdk/props/ApiResponse.dart';
import 'package:ui_sdk/utils/pubsub/localpubsub.dart';
import 'package:flutter/material.dart';
import 'package:ui_sdk/utils/update_ui_provider.dart';
import 'Renderer.dart';
import 'utils/init_data.dart';
import 'package:http/http.dart' as http;

abstract class StandardPage {
  late StatefulWidget page;

  dynamic getData();
  Set<String?> getActions();
  dynamic getPageCallBackForAction();
  dynamic getPageContext();
  dynamic getConstructorArgs();

  StandardPage([InitData? args]) {
    page = FlutterPage(
      getData(),
      getActions(), // set of strings
      getPageCallBackForAction(),
      /*
        ........................Syntax..............................

        getPageCallBackForAction() {
          return callback();
        }

        void callback() {

        }
      */
      getPageContext(),

      getConstructorArgs(),
      /*
        getPageContext() {
          return pageReference;
        }
      */
      args: args,
    );
  }

  StatefulWidget getPage() {
    return page;
  }
}

class FlutterPage extends StatefulWidget {
  dynamic apiData;
  Set<String?> actionList;
  dynamic fun;
  dynamic reference;
  Map<String?, Subscription?>? subArray;
  dynamic constructorArgs;
  InitData? args;

  FlutterPage(this.apiData, this.actionList, this.fun, this.reference,
      this.constructorArgs,
      {Key? key, this.args})
      : super(key: key);

  @override
  State<FlutterPage> createState() => FlutterPageState();
}

class FlutterPageState extends State<FlutterPage> {
  Renderer renderer = Renderer();

  @override
  void initState() {
    super.initState();
    registerEvents();
  }

  registerEvents() {
    //we have to unsubscribe the elements here
    //but for now we don't have to do right now
    widget.subArray = Map(); //string,Subscription
    PubSub pubsub = PubSub();
    //set
    widget.actionList.forEach(
      (element) {
        Subscription? sub;
        sub = pubsub.subscribe(element, widget.reference);

        // print(widget.subArray);
        widget.subArray![element] = sub;
        //  print(widget.subArray);

        sub?.stream?.listen(
          (uikAction) {
            // print(uikAction);
            // print(pubsub.stack.top());

            //because the last reference will always be of Prateek
            //   print("sub");
            //   print(" ${widget.reference}  ${pubsub.stack.top()} ");

            if (widget.reference == pubsub.stack.top()) {
              // if (uikAction.tap.type == "UPDATE_UI") {
              //   Provider.of<UpdateUIProvider>(context, listen: false).updateUI();
              // }
              widget.fun(uikAction);
            }
          },
        );
      },
    );
    // print(widget.subArray);
    //   print("widget.reference ${widget.reference}");
    pubsub.stack.push(widget.reference);
    // print(pubsub.stack.length);
    //print(widget.reference);
  }

  void rebuildUI() {
    setState(() {});
  }

  @override
  Widget build(BuildContext context) {
    //extracting args
    final dynamic args = ModalRoute.of(context)!.settings.arguments ?? {};
    final dynamic constructorArguments = widget.constructorArgs;

    // print("args ${args}");
    // print("constructor args ${constructorArguments}");
    // final totalArgs = [... args, ...constructorArguments];
    // print("total args ${totalArgs}");
    Map finalArgs = {...constructorArguments, ...args};
    var ispullRefresh = true;
    // fetchAlbum(args);
    void flutterPageSetState() {
      setState(() {});
    }

    return WillPopScope(
      onWillPop: () async {
        PubSub pubsub = PubSub();
        // if (pubsub.stack.length > 0) {
        //   print(" before pop ${pubsub.stack.length} -  ${pubsub.stack.top()}");
        // }
        pubsub.stack.pop();
        // if (pubsub.stack.length > 0) {
        //   print(" after pop ${pubsub.stack.length} - ${pubsub.stack.top}");
        // }
        pubsub.unsubscribe(widget.subArray);
        return true;
      },
      child: MaterialApp(
        debugShowCheckedModeBanner: false,
        title: 'Fetch Data Example',
        theme: ThemeData(
          primarySwatch: Colors.blue,
        ),
        home: Scaffold(
          body: Center(
            child: FutureBuilder<ApiResponse>(
              future: widget.apiData(finalArgs),
              builder: (context, snapshot) {
                if (snapshot.hasData) {
                  print("coming here");
                  return renderer.getListWidgetUi(
                      StandardScreenResponse.fromJson(
                          snapshot.requireData.data),
                      rebuildUI,
                      context);
                } else if (snapshot.hasError) {
                  return Text('${snapshot.error}');
                }

                // By default, show a loading spinner.
                return const CircularProgressIndicator(
                  color: Colors.yellow,
                );
              },
            ),
          ),
        ),
      ),
    );
  }
}
